#!/usr/bin/perl -w
###############################################################################
#
# Freenet CGI Request Client v0.0.6
#
# Copyright (C) 1999, 2000 Steven Hazel
#
# Contributions on May 26, 2000 by Lee Benjamin Burhans
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# see http://freenet.sourceforge.net/ for more info on Freenet
#
###############################################################################

use strict;
use freenet;
use keyindex;
use CGI qw(:standard);
use LWP::Simple;

###############################################################################

my $DEFAULTNODE     = "localhost";                  # def node host 
my $DEFAULTPORT     = 19114;                        # def node port
my $DEFAULTHOPS     = 5;                            # def hops for DataRequest
my $DEFAULTDL       = 5;                            # low value for depth
my $DEFAULTDH       = 30;                           # high value for depth
my $DEFAULTHASH     = "SHA1";                       # hash type
my $DEFAULTINDEXFILE = "";                          # local key index
my $DEFAULTKEYINDEX = "";                           # remote key index

my ($node, $port, $key, $keyindex, $hop, $status, $line, $sock);

###############################################################################

$node = $DEFAULTNODE;                        # node host to contact 
$port   = param('port')     || $DEFAULTPORT;   # node port to contact
$hop    = param('hop')      || $DEFAULTHOPS;   # hops for DataRequest
$keyindex = param('keyindex') || "";           # remote key index
$key    = param('key') || $ENV{'PATH_INFO'} || &blurb();    # key to request

$key =~ s/^\///;                                    # kill leading "/"
$key =~ tr/+/ /;                                    # replace "+" with spaces
$key =~ s/%([a-fA-F0-9]{2})/pack("C", hex($1))/eg;  # decode hex

$key || blurb();                               # display blurb if no key left

freenet::configure($node, $port, $DEFAULTDH, $DEFAULTDL, $hop, $DEFAULTHASH);

($status, $sock) = freenet::request($key);

if ($status ne "Success") {

  if (($status eq "Requested key not found.") 
      or ($status eq "Request timed out.")) {
    # temporary: cull the key from the key index
    if ($DEFAULTINDEXFILE ne "") {
      keyindex::del_key($DEFAULTINDEXFILE, $key);
    }
  }

  print "Content-type: text/html\n\n";
  print $status;

} else {
  #
  # print contents of found file
  #

  print "Content-type: " . get_content_type($key) . "\n\n";

  while ($line = <$sock>) {
    print $line;
  }

  close($sock);

  # temporary: add the key to the key index
  if (($DEFAULTINDEXFILE eq "") or ($keyindex ne "")) {
    if ($keyindex eq "") {
      $keyindex = $DEFAULTKEYINDEX;
    }
    if (get($keyindex) =~ m/yes/) {
      get("$keyindex?key=$key");
    }
  } elsif ($DEFAULTINDEXFILE ne "") {
    keyindex::add_key($DEFAULTINDEXFILE, $key);
  }
}

exit;


sub blurb() {
  print "Content-type: text/html\n\n";
  print "<title>FCRC</title>\n";
  print p("Freenet CGI Request Client v0.0.6");
  print p("Copyright (C) 1999, 2000 Steven Hazel.");
  print "<form ";
  print "action=../keydir/ ";
  print "method=get>\n";
  print "keyindex: <input type=textarea name=keyindex><br>\n";
  print "search: <input type=textarea name=search><input type=submit>\n";
  print "</form>\n";

  exit;
}


sub get_content_type() {
  my($key)    = @_;
  my($type)   = ();
  my($def_ct) = "text/plain";

  # Try not to gag, this is temporary.  I need metadata.
  my(%cont_types) = ("\.html" => "text/html",
		     "\/"     => "text/html",
		     "\.txt"  => "text/plain",
		     "\.gz"   => "application/gzip",
		     "\.tgz"  => "application/gzip",
		     "\.z"    => "application/x-compress",
		     "\.zip"  => "application/x-Zip",
		     "\.jar"  => "application/x-Zip",
		     "\.jpg"  => "image/jpeg",
		     "\.jpeg" => "image/jpeg",
		     "\.gif"  => "image/gif",
		     "\.png"  => "image/png",
		     "\.mp3"  => "audio/x-mpeg3"
		    );

  foreach $type (keys %cont_types) {
    if ($key =~ m/$type$/i) {
      return $cont_types{$type};
    }
  }    

  return $def_ct;
}
